@@ -37,6 +37,10 @@
 
 #include <sdcc-lib.h>
 
+#ifndef EOF
+  #define EOF (-1)
+#endif
+
 #ifndef NULL
   #define NULL (void *)0
 #endif
