@@ -260,9 +260,8 @@ _divslong (long x, long y)
 {
   long r;
 
-  r = _divulong((x < 0 ? -x : x),
-                (y < 0 ? -y : y));
-  if ( (x < 0) ^ (y < 0))
+  r = (unsigned long)(x < 0 ? -x : x) / (unsigned long)(y < 0 ? -y : y);
+  if ((x < 0) ^ (y < 0))
     return -r;
   else
     return r;
