@@ -14,7 +14,7 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License 
+   You should have received a copy of the GNU General Public License
    along with this library; see the file COPYING. If not, write to the
    Free Software Foundation, 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301, USA.
@@ -27,7 +27,7 @@
    might be covered by the GNU General Public License.
 -------------------------------------------------------------------------*/
 
-#include "string.h" 
+#include <string.h> 
 
 #undef memset /* Avoid conflict with builtin memset() in Z80 and some related ports */
 
@@ -181,4 +181,3 @@ void *memset (void *s, int c, size_t n)
   }
 
 #endif
-
