@@ -260,8 +260,8 @@ _modslong (long a, long b)
 {
   long r;
 
-  r = _modulong((a < 0 ? -a : a),
-                (b < 0 ? -b : b));
+  r = (unsigned long)(a < 0 ? -a : a) % (unsigned long)(b < 0 ? -b : b);
+
   if (a < 0)
     return -r;
   else
