@@ -629,7 +629,7 @@ struct some_struct {
 	short a ;
 	char b;
 	long c ;};
-#if defined(__SDCC_hc08) || defined(__SDCC_s08)
+#if defined(__SDCC_hc08) || defined(__SDCC_s08) || defined(__SDCC_stm8)
 /* big endian order */
 union bil {
         struct {unsigned char b3,b2,b1,b0 ;} b;
@@ -698,6 +698,37 @@ _mullong (long a, long b)
 
         return t.l;
 }
+#elif defined(__SDCC_z80) || defined(__SDCC_gbz80) || defined(__SDCC_r2k) || defined(__SDCC_r3k)
+/* 32x32->32 multiplication to be used
+   if 16x16->16 is faster than three 8x8->16.
+   2009, by M.Bodrato ( http://bodrato.it/ )
+
+   z80 and gbz80 don't have any hardware multiplication.
+   r2k and r3k have 16x16 hardware multiplication.
+ */
+long
+_mullong (long a, long b)
+{
+  unsigned short i12;
+
+  bcast(a)->i.hi *= bcast(b)->i.lo;
+  bcast(a)->i.hi += bcast(b)->i.hi * bcast(a)->i.lo;
+
+  /* only (a->i.lo * b->i.lo) 16x16->32 to do. asm? */
+  bcast(a)->i.hi += bcast(a)->b.b1 * bcast(b)->b.b1;
+
+  i12 = bcast(b)->b.b0 * bcast(a)->b.b1;
+  bcast(b)->bi.i12 = bcast(a)->b.b0 * bcast(b)->b.b1;
+
+  /* add up the two partial result, store carry in b3 */
+  bcast(b)->b.b3 = ((bcast(b)->bi.i12 += i12) < i12);
+
+  bcast(a)->i.lo  = bcast(a)->b.b0 * bcast(b)->b.b0;
+
+  bcast(b)->bi.b0 = 0;
+
+  return a + b;
+}
 #else
 long
 _mullong (long a, long b)
