@@ -13,7 +13,7 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License 
+   You should have received a copy of the GNU General Public License
    along with this library; see the file COPYING. If not, write to the
    Free Software Foundation, 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301, USA.
@@ -47,7 +47,7 @@
    #define myconst const
 #endif
 
-float asincosf(const float x, const int isacos)
+float asincosf(float x, int isacos)
 {
     float y, g, r;
     int i;
