@@ -13,7 +13,7 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License 
+   You should have received a copy of the GNU General Public License
    along with this library; see the file COPYING. If not, write to the
    Free Software Foundation, 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301, USA.
@@ -50,7 +50,7 @@
    #define myconst const
 #endif
 
-float atanf(const float x) _MATH_REENTRANT
+float atanf(float x) _MATH_REENTRANT
 {
     float f, r, g;
     int n=0;
