@@ -13,7 +13,7 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License 
+   You should have received a copy of the GNU General Public License
    along with this library; see the file COPYING. If not, write to the
    Free Software Foundation, 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301, USA.
@@ -42,7 +42,7 @@
 #define C1  0.693359375 /*355.0/512.0*/
 #define C2 -2.121944400546905827679E-4
 
-float logf(const float x) _MATH_REENTRANT
+float logf(float x) _MATH_REENTRANT
 {
 #if     defined(SDCC_mcs51) && defined(SDCC_MODEL_SMALL) \
     && !defined(SDCC_NOOVERLAY)
