@@ -54,7 +54,7 @@
  *
  */
 
-/* sdld 8051 & 6808 specific */
+/* sdld 8051 specific */
 /*JCF:  Creates some of the default areas so they are allocated in the right order.*/
 void Areas51 (void)
 {
@@ -116,14 +116,12 @@ void Areas51 (void)
                 }
         }
 
-        if (TARGET_IS_8051) {
-                sp = lkpsym("l_IRAM", 1);
-                sp->s_addr = ((iram_size>0) && (iram_size<=0x100)) ? iram_size : 0x0100;
-                sp->s_axp = NULL;
-                sp->s_type |= S_DEF;
-        }
+        sp = lkpsym("l_IRAM", 1);
+        sp->s_addr = ((iram_size>0) && (iram_size<=0x100)) ? iram_size : 0x0100;
+        sp->s_axp = NULL;
+        sp->s_type |= S_DEF;
 }
-/* end sdld 8051 & 6808 specific */
+/* end sdld 8051 specific */
 
 /*)Function     int     main(argc,argv)
  *
@@ -221,7 +219,7 @@ main(int argc, char *argv[])
         /* sdas initialization */
         sdld_init(argv[0]);
 
-        /* use these defaults for parsing the .lk script */
+        /* use these defaults for parsing the .lnk script */
         a_bytes = 4;
         a_mask = 0xFFFFFFFF;
         s_mask = 0x80000000;
@@ -338,7 +336,7 @@ main(int argc, char *argv[])
                 radix = 10;
 
                 /* sdld specific */
-                if (TARGET_IS_8051 || TARGET_IS_6808)
+                if (TARGET_IS_8051)
                         Areas51(); /*JCF: Create the default 8051 areas in the right order*/
                 /* end sdld specific */
 
@@ -627,8 +625,8 @@ link_main()
                         a_mask = 0xFFFFFFFFl;
                         s_mask = 0x80000000l;
                         v_mask = 0x7FFFFFFFl;
-                                break;
-                        }
+                        break;
+                }
 #else
                 switch(a_bytes) {
                 default:
