@@ -41,6 +41,8 @@ static OPTION _pic14_poptions[] =
     { 0, "--debug-xtra",   &debug_verbose, "show more debug info in assembly output" },
     { 0, "--no-pcode-opt", &pic14_options.disable_df, "disable (slightly faulty) optimization on pCode" },
     { 0, OPTION_STACK_SIZE, &options.stack_size, "sets the size if the argument passing stack (default: 16, minimum: 4)", CLAT_INTEGER },
+    { 0, "--no-extended-instructions", &pic14_options.no_ext_instr, "forbid use of the extended instruction set (e.g., ADDFSR)" },
+    { 0, "--no-warn-non-free", &pic14_options.no_warn_non_free, "suppress warning on absent --use-non-free option" },
     { 0, NULL, NULL, NULL }
   };
 
@@ -161,6 +163,17 @@ _pic14_finaliseOptions (void)
       dbuf_append (&dbuf, upperProc, len);
       addSet (&preArgvSet, dbuf_detach_c_str (&dbuf));
     }
+
+  if (!pic14_options.no_warn_non_free && !options.use_non_free)
+    {
+      fprintf(stderr,
+              "WARNING: Command line option --use-non-free not present.\n"
+              "         When compiling for PIC14/PIC16, please provide --use-non-free\n"
+              "         to get access to device headers and libraries.\n"
+              "         If you do not use these, you may provide --no-warn-non-free\n"
+              "         to suppress this warning (not recommended).\n");
+    } // if
+
 }
 
 static void
@@ -341,8 +354,6 @@ PORT pic_port =
     NULL,
     "-g",           /* options with --debug */
     NULL,           /* options without --debug */
-    //"-plosgffc",  /* Options with debug */
-    //"-plosgff",   /* Options without debug */
     0,
     ".asm",
     NULL            /* no do_assemble function */
@@ -424,6 +435,7 @@ PORT pic_port =
   _pic14_setDefaultOptions,
   pic14_assignRegisters,
   _pic14_getRegName,
+  NULL,
   _pic14_keywords,
   _pic14_genAssemblerPreamble,
   NULL,         /* no genAssemblerEnd */
