@@ -41,6 +41,7 @@ extern char *VersionString;
 extern struct dbuf_s *codeOutBuf;
 extern char *iComments1;
 extern char *iComments2;
+extern int has_xinst_config;
 
 extern int initsfpnt;
 extern unsigned long pFile_isize;
@@ -792,9 +793,9 @@ pic16_printIvalBitFields (symbol **sym, initList **ilist, char ptype, void *p)
   symbol *lsym = *sym;
   initList *lilist = *ilist;
   unsigned long ival = 0;
-  int size = 0;
+  unsigned size = 0;
   int bit_start = 0;
-  int i;
+  unsigned i;
 
 
 #if DEBUG_PRINTIVAL
@@ -804,7 +805,7 @@ pic16_printIvalBitFields (symbol **sym, initList **ilist, char ptype, void *p)
 
   while (lsym && IS_BITFIELD (lsym->type))
     {
-      int bit_length = SPEC_BLEN (lsym->etype);
+      unsigned bit_length = SPEC_BLEN (lsym->etype);
       if (0 == bit_length)
         {
           /* bit-field structure member with a width of 0 */
@@ -1761,6 +1762,14 @@ pic16glue ()
 
     mainf = findSymWithLevel(SymbolTab, mainf);
 
+    if (mainf && IFFUNC_HASBODY(mainf->type) && pic16->xinst && !has_xinst_config)
+      {
+        fprintf(stderr, "WARNING: The target device seems to support XINST and no #pragma config XINST=OFF was found.\n");
+        fprintf(stderr, "         The code generated by SDCC does probably not work when XINST is enabled (possibly by default).\n");
+        fprintf(stderr, "         Please make sure to disable XINST.\n");
+        fprintf(stderr, "         (If the target does not actually support XINST, please report this as a bug in SDCC.)\n");
+      } // if
+
     pic16_pCodeInitRegisters();
 
     if(pic16_options.no_crt && mainf && IFFUNC_HASBODY(mainf->type)) {
